<?php

require "config.php";
require "standards.php";
require "domtools.php";

find_portal_root();
$this_user = check_auth_user();
$look_for = "xs:anyURI";
$suffix_regexp = preg_replace("/\./", "\.", $xml_suffix);
//$this_debug = 1;

function find_schemas () {
    global $xsd_path, $doc_root;
    $port_xsd_path = $_SERVER['DOCUMENT_ROOT'] . "/$doc_root/$xsd_path";
    $schema_dirs = array($xsd_path, $port_xsd_path);
    $schemas = array();

    foreach ($schema_dirs as $path) {
	if ($dir = @opendir($path)) {
	    while ($entry = readdir($dir)) {
		$file = "$path/$entry";
		if (is_dir($file) or neglect_file($file)) {
		    continue;
		}
		if (!preg_match("/\.xsd$/", $file)) {
		    continue;
		}
		$schemas[$entry] = $file;
	    }
	    closedir($dir);
	} else {
	    exception("Kunne ikke bne katalog '$path'");
	    die;
	}
    }
    return $schemas;
}


function find_url_elem ($xsd) {
    if (!($dom = domxml_open_file($xsd))) {
	exception("Kunne ikke parse '$xsd'");
	die;
    }
    $root = $dom->document_element();
    $elements = array();
    $path = array();
    xsd_parse($root, $elements, $path);
    return $elements;
}


function xsd_parse ($node, &$elements, &$path) {
    global $look_for;
    
    if (is_array($kids = $node->child_nodes())) {
	foreach ($kids as $kid) {
	    if ($tag = $kid->tagname) {
		$name = $kid->get_attribute("name");
		if ($tag == "element" or $tag == "attribute") {
		    if ($name) {
			if ($tag == "attribute") {
			    $name = "@$name";
			}
			$ext_path = array_merge($path, $name);
			if ($type = $kid->get_attribute("type")) {
			    if ($type == $look_for) {
				$elements[] = $ext_path;
			    }
			} else {
			    xsd_parse($kid, $elements, $ext_path);
			}
		    }
		} elseif ($tag == "sequence") {
		    xsd_parse($kid, $elements, $path);
		} elseif ($tag == "complexType") {
		    xsd_parse($kid, $elements, $path);
		} elseif ($tag == "simpleType") {
		    xsd_parse($kid, $elements, $path);
		} elseif ($tag == "restriction") {
		    if ($base = $kid->get_attribute("base")) {
			if ($base == $look_for) {
			    $elements[] = $path;
			}
		    } else {
			exception("xs:restriction without base attribute");
		    }
		} elseif ($tag == "annotation") {
		} elseif ($tag == "simpleContent") {
		    // This one should be supported as well...
		} else {
		    echo "<b>Advarsel: Tagget '$tag' er ikke understttet i parseren</b><br>";
		}
	    }
	}
    }
}


function browse_dir ($dir, &$elements, $xsd, &$records) {
    global $this_debug, $suffix_regexp;
    
    if ($dh = @opendir($dir)) {
	while ($entry = readdir($dh)) {
	    $file = "$dir/$entry";
	    if (neglect_file($entry)) {
		continue;
	    } elseif (is_dir($file)) {
		browse_dir($file, $elements, $xsd, $records);
	    } elseif (!preg_match("/$suffix_regexp/", $entry)) {
		continue;
	    } else {
		if ($schema = extract_schema($file)) {
		    if ($schema != $xsd) {
			continue;
		    }
		    if (!($dom = domxml_open_file($file))) {
			exception("Kunne ikke parse '$file'");
			die;
		    }
		    $root = $dom->document_element();
		    $creator = $root->get_attribute('creator');
		    if ($this_debug) {
			echo "TAG='", $root->tagname, "' FILE='$file' SCHEMA='$xsd'<br>";
		    }
		    foreach ($elements as $xpath) {
			array_shift($xpath);
			if ($nodes = dom_select_ext($root, $xpath)) {
			    foreach ($nodes as $node) {
				$url = preg_replace("/&/", "&amp;", get_content($node));
				//$url = $node->get_content();
				if ($this_debug) {
				    echo "NODE_TAG='", $node->tagname, "' URL='$url' CREATOR='$creator'<br>";
				}
				$records[] = array("URL"=>$url, "FILENAME"=>$file, "USER"=>$creator);
			    }
			}
		    }
		} else {
		    exception("Filen '$file' havde ikke nogen schema oplysninger");
		    die;
		}
	    }
	}
	closedir($dh);
    } else {
	exception("Kataloget '$dir' kunne ikke bnes");
	die;
    }
}
	    




$schemas = find_schemas();
$matching = array();
foreach ($schemas as $schema => $filename) {
    $elements = find_url_elem($filename);
    if (count($elements)) {
	browse_dir($abs_cwd, $elements, $schema, $matching);
    }
}
$portal_root = $_SERVER['DOCUMENT_ROOT'] . "/$doc_root";
if (!($fh = fopen("$portal_root/$url_list_file", "w"))) {
    exception("Kunne ikke bne checkurl xml-fil '$url_list_file'");
    die;
}
fputs($fh, "$xml_header\n");
fputs($fh, "<URLS>\n");
foreach ($matching as $info) {
    if ($this_debug) {
	echo "FILE='", $info['FILENAME'], "' URL='", $info['URL'], "' USER='", $info['USER'], "'<br>";
    }
    fputs($fh, "  <CHECKURL>\n");
    foreach ($info as $key => $value) {
	fputs($fh, "    <$key>$value</$key>\n");
    }
    fputs($fh, "  </CHECKURL>\n");
}
fputs($fh, "</URLS>\n");
fclose($fh);
$path = dirname($_SERVER['SCRIPT_NAME']);
$command = "wget \"http://".$_SERVER['HTTP_HOST'].$path."/robot.php?cwd=$cwd\"";
system ($command." &> robot_output.txt");
// $Log: get_urls.php,v $
// Revision 1.12  2002/10/24 12:07:33  quinn
// Correction in order to make robot.php run autmaticaly
//
// Revision 1.11  2002/10/24 10:24:26  perhans
// Robot.php now runs automatically after ge_urls.php has finished
//
// Revision 1.10  2002/10/23 09:57:02  sondberg
// The creator field is extracted from each record with url-fields, and stored in urls.tkl.
//
// Revision 1.9  2002/09/13 08:53:12  adam
// Compatible DOMXML (PHP 4.1, 4.2 and 4.3 shold work)
//
// Revision 1.8  2002/07/23 08:55:01  sondberg
// Ampersands are escaped (again).
//
// Revision 1.7  2002/07/22 13:59:39  sondberg
// Removed & escaping again. Should be handled differently.
//
// Revision 1.6  2002/07/22 13:52:31  sondberg
// Escaping ampersand.
//
// Revision 1.5  2002/07/12 10:03:37  sondberg
// Now the urls.xml file is created in the portal root.
//
// Revision 1.4  2002/07/12 10:00:06  sondberg
// Now this script creates a file with urls and corresponding file names.
//
// Revision 1.3  2002/07/10 14:00:43  sondberg
// Corrected a bug.
//
// Revision 1.2  2002/07/10 13:34:38  sondberg
// simpleContent.
//
// Revision 1.1  2002/07/10 13:31:44  sondberg
// Added this file to repository.
//

?>
