<?php

// Copyright (c) 2002 Index Data.
// Authors: Anders Snderberg Mortensen and Sebastian Hammer


$metadataPrefix = $_REQUEST['metadataPrefix'];

// Here goes the valid verbs as well as the additional parameters, mandatory (1) as well as optional (2)
$verbs = array(
    "ListRecords"	  => array("metadataPrefix"=>1, "from"=>2, "until"=>2, "set"=>2),
    "Identify"		  => array(),
    "GetRecord"		  => array("metadataPrefix"=>1, "identifier"=>1),
    "ListIdentifiers"	  => array("metadataPrefix"=>1, "from"=>2, "until"=>2, "set"=>2),
    "ListMetadataFormats" => array("identifier"=>2),
    "ListSets"		  => array(),
);

// These are the supported metadata formats
$metadataformats = array(
    "oai_dc"		=> array(
	"schema"	=> "http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
	"namespace"	=> "http://www.openarchives.org/OAI/2.0/oai_dc/",
    )
);

// The XML-header
$xml_header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";

// The OAI-PMH root tag should like this according to OAI/v2
$oai_pmh = <<<END
<OAI-PMH xmlns="http://www.openarchives.org/OAI/2.0/"
       	 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	 xsi:schemaLocation="http://www.openarchives.org/OAI/2.0/
         http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd">
END;

$repositoryName = 'Unknown';
$adminEmail = 'Unknown@unknown.dk';

function do_oai_verb($verb, $dir) {
    global $verbs;
    global $oai_pmh;
    global $xml_header;
    global $oai_base_url;
    global $repositoryName;
    global $adminEmail;

    $oai_base_url = "http://" .
	$_SERVER['SERVER_NAME'] .
	($_SERVER['SERVER_PORT'] != 80 ? (":" . $_SERVER['SERVER_PORT']) : '').
	$dir;

    //print "DO_OAI: v=$verb, d=$dir";

    if (!file_exists('tkl.config')) {
	header("HTTP/1.0 400 OAI Requests must be directed at portal root.");
	echo("HTTP/1.0 400 OAI Requests must be directed at portal root.");
	die;
    }

    if ($d = domxml_open_file('index.tkl')) {
	$doc = $d->document_element();
	if (list($title) = $doc->get_elements_by_tagname('name')) {
	    $text= $title->first_child();
	    $repositoryName = $text->node_value();
	}
	if (list($creator) = $doc->get_elements_by_tagname('creator')) {
	    $adminEmail = $creator->get_attribute('email');
	}
    }

    if (is_array($required = $verbs[$verb]) && function_exists($verb)) {
	foreach ($required as $p => $type) {		// Check that required arguments are there
	    if (($type == 1) && empty($_REQUEST[$p])) {
		header("HTTP/1.0 400 Malformed OAI request. Missing $p.");
		echo("HTTP/1.0 400 Malformed OAI request. Missing $p.");
		die;
	    }
	}
	header("Content-Type: text/xml");
	echo $xml_header, "\n", $oai_pmh, "\n";
	echo "<responseDate>", get_utc(), "</responseDate>\n";
	echo "<request" , make_attr(make_attr_list()), ">$oai_base_url</request>\n";
	$verb();
	echo "</OAI-PMH>\n";
	die;
    } else {
	header("HTTP/1.0 400 Malformed request, verb $verb not supported.");
	echo("HTTP/1.0 400 Malformed request, verb $verb not supported.");
	die;
    }
}

function dump_record($path) {
    global $metadataPrefix;
    global $repositoryName;

    if (!($f = @file($path)))
	return;
    $rec = join('', $f);
    $t = new XML_getschema($rec);
    $schema = $t->schema();
    $confiltername = "$schema--$metadataPrefix.xsl";
    if (file_exists("schemas/$confiltername")) {
	$xsl = xslt_create();
	$args = array('/_xml' => $rec);
	$result = xslt_process($xsl, 'arg:/_xml', "schemas/$confiltername",
	    NULL, $args);
	$result = preg_replace("/<\?.*?\?>\s*/", "", $result);

	if (preg_match("/<nothing/", $result))
	    return;

	print "<record>\n";

	print "<header>\n";
	print "<identifier>oai:$repositoryName.tkl.indexdata.com:" .
	    preg_replace("'^\./'", "", $path) . "</identifier>\n";
	$stat = stat($path);
	$datestamp = get_utc($stat[10]);
	print "<datestamp>$datestamp</datestamp>\n";
	print "</header>\n";

	print "<metadata>\n";

	print "$result";

	print "</metadata>\n";

	print "</record>\n";
    }
}

function dump_records($dir, $active=0) {
    $path = $dir ? $dir : ".";

    if ($dir_arr = @file("$path/directory.tkl")) {
	$dir_r = join('', $dir_arr);
	if ($v = preg_match("/<searchable>\s*(.*?)\s*<\/searchable>/", $dir_r,
	    $match) && $match[1])
	    $active = 1;

    }
    $d = opendir($path);
    while (false != ($file = readdir($d))) {
        if (preg_match("/^\./", $file))
	    continue;
        if (is_dir("$path/$file")) {
	    dump_records("$path/$file", $active);
	}
	elseif (preg_match("/.*.\.tkl$/", $file) && $active) {
	    dump_record("$path/$file");
	}
    }
}

function ListRecords () {
    global $metadataPrefix;

    echo "<ListRecords>\n";
    dump_records("");
    echo "</ListRecords>\n";
}


function Identify () {
    global $oai_base_url;
    global $repositoryName;
    global $adminEmail;


    echo "<Identify>\n";
    echo "<repositoryName>$repositoryName</repositoryName>\n";
    echo "<baseURL>$oai_base_url</baseURL>\n";
    echo "<protocolVersion>2.0</protocolVersion>\n";
    echo "<adminEmail>$adminEmail</adminEmail>\n";
    echo "<earliestDatestamp>1978-01-01T12:00:00Z</earliestDatestamp>\n";
    echo "<deletedRecord>no</deletedRecord>\n";
    echo "<granularity>YYYY-MM-DDThh:mm:ssZ</granularity>\n";
    echo <<<END
<description>
  <oai-identifier xmlns="http://www.openarchives.org/OAI/2.0/oai-identifier"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://www.openarchives.org/OAI/2.0/oai-identifier http://www.openarchives.org/OAI/2.0/oai-identifier.xsd">
    <scheme>oai</scheme> 
END;
    echo "<repositoryIdentifier>$repositoryName.tkl.indexdata.com</repositoryIdentifier>";
    echo "<delimiter>:</delimiter>";
    echo "<sampleIdentifier>oai:$repositoryName.tkl.indexdata.com:links/stuff.tkl</sampleIdentifier>";
    echo "</oai-identifier>";
    echo "</description>";

    echo "</Identify>\n";
}


function GetRecord () {
    global $metadataPrefix, $identifier;

    echo "<GetRecord>\n";
    echo "GetRecord currently not implemented";
    echo "</GetRecord>\n";
}

function ListIdentifiers () {
    global $metadataPrefix;
    
    echo "<ListIdentifiers>\n";
    echo "ListIdentifier currently not implemented";
    echo "</ListIdentifiers>\n";
}

function ListSets () {
    echo "<error code=\"noSetHierarchy\">This repository does not support sets</error>";
}


function ListMetadataFormats () {
    global $metadataformats, $identifier;

    echo "<ListMetadataFormats>\n";
    if (isset($identifier)) {		// Find available metadataformats for this specific record
        print "<error>Can't do that yet</error>";
    } else {
        $d = opendir("schemas");
	while (false != ($f = readdir($d))) {
	    if (preg_match("/^(.*?)--(.*?)\.xsl$/", $f, $match)) {
		$prefix = $match[2];
		echo "<metadataFormat>\n";
		echo "<metadataPrefix>$prefix</metadataPrefix>\n";
		$ss = join('', file("schemas/$f"));
		// Hack to acquire schema definition: Look for first tag
		// with the right prefix
		if (preg_match("/<$prefix" . "[^>]*xsi:schemaLocation=\"(.*?)\"/s", $ss, $match)) {
		    if (preg_match("/(.*?)\s+(.*)/s", $match[1], $smatch)) {
			print "<schema>$smatch[2]</schema>\n";
			print "<metadataNamespace>$smatch[1]</metadataNamespace>";
		    }
		}
		echo "</metadataFormat>\n";
	    }
	}
	closedir($d);
    }
    echo "</ListMetadataFormats>\n";
}


function get_utc ($time='') {
    if (!$time)
	$time = time();

    $epoch = $time - date("Z");	// The current Unix time-stamp with respect to UTC

    return date("Y-m-d", $epoch) . "T" . date("H:i:s", $epoch) . "Z";
}


function make_attr ($attr) {
    $ret = "";
    foreach ($attr as $name => $value) {
	if ($value)
	    $ret .= " $name=\"$value\"";
    }
    return $ret;
}


function make_attr_list () {
    global $verbs;
    $list = array('verb'=>$_REQUEST['verb']);

    foreach ($verbs[$_REQUEST['verb']] as $name => $dummy) {
	$list[$name] = $_REQUEST[$name];
    }
    return $list;
}
	
?>
