<?php

// Copyright (c) 2002 Index Data.
// Author: Sebastian Hammer

setlocale(LC_ALL, 'da_DK');

if ($_COOKIE['tkl_admin'] == 1)
    $tkl_authenticated = 1;
else
    $tkl_authenticated = "";

$path = $_SERVER['PATH_TRANSLATED'];

$blacklist = array('directory.tkl' => 1, 'users.tkl' => 1);
$fn = preg_replace("'^.*/'", "", $path);
if ($blacklist[$fn]) {
    header("HTTP/1.1 403 Forbidden");
    print "Access denied.";
    die;
}

include 'shell-util.php';
include 'oai.php';

$soappath='soap/nusoap-0.6.1/nusoap.php';
if (file_exists($soappath))
    require 'soap/nusoap-0.6.1/nusoap.php';

$debug = $_REQUEST['debug'];

if (!($fl = @file($path))) {
    header("HTTP/1.1 404 Not Found");
    print "File not found";
    die;
}

$document = join('', file($path));
$workingdir = preg_replace('/\/[^\/]*$/', "", $_SERVER['PATH_TRANSLATED']);
chdir($workingdir);

// Hidden attribute set
if (preg_match("/<[^?!][^>]*hidden=.1./", $document)) {
    header("HTTP/1.1 403 Forbidden");
    print "Access denied.";
    die;
}

if ($_REQUEST['verb']) {
    do_oai_verb($_REQUEST['verb'], $_SERVER['PATH_INFO']);
    return;
}

session_start();

if ($_REQUEST['lang']) {
    $lang = $_REQUEST['lang'];
    $_SESSION['lang'] = $lang;
} elseif ($_SESSION['lang'])
    $lang = $_SESSION['lang'];
else
    $lang = 'da';

$t = new XML_getschema($document);
if ($debug) print "Schema is " . $t->schema() . "<p>\n";
$schema = $t->schema();
if (!$schema)
    $schema = "dummy";

$t = new XML_process_stylesheet($document);
$document = $t->buf();

if ($debug) {
    print "DOCUMENT:";
    print "<pre>"; print preg_replace("/</", "&lt;", $document); print "</pre>";
    print "<hr>";
    print "STYLESHEET";
    print "<pre>"; print preg_replace("/</", "&lt;", $stylesheet); print "</pre>";
}


// Use trailing part of schema to select stylesheet.
$spart = preg_replace("/^.*[:\/]/", "", $schema);

$root = preg_replace('/\/[^\/]*$/', "", $_SERVER['PATH_INFO']);

// Look for stylesheet from here back to the root
// Is there a prettier way to do this, please????
$dir = $workingdir;
$stylesheet = '';

do {
    if (!$stylesheet && file_exists("$dir/$spart.xsl")) {
        $stylesheetpath = "$dir/$spart.xsl";
	$stylesheet = join('', file($stylesheetpath));
    }
    if (file_exists("$dir/tkl.config"))
        break;
    $dir = preg_replace("/\/[^\/]*$/", "", $dir); // up one dir
    $root = preg_replace("/\/[^\/]*$/", "", $root); // up one dir
} while ($dir);

//
// ============= Pre-process the stylesheet
// == NOTE: THe PORTCOM substitution commands are being phased out in favor of accessing
// == extra functionality through custom URI schemes. However, we still need the preproc
// == stage to filter out language-specific elements.
//

if (!$stylesheet) {
    print "<pre>"; print preg_replace("/</", "&lt;", $document); print "</pre>";
    exit;
}

// Execute &lt;? PHP-code ?&gt; expressions.

function execute_script($matches) {
    return eval ("$matches[1];");
}

$stylesheet = preg_replace_callback('/<\?[pP][Hh][Pp]\s*(.*?)\s*\?>/s', 'execute_script',
    $stylesheet);

function lookup_var($matches) {
    return $_REQUEST[$matches[1]];
}

$stylesheet = preg_replace_callback('/\$WEB\[\'(.*?)\'\]/', 'lookup_var',
    $stylesheet);

function lookup_global($matches) {
    return $GLOBALS[$matches[1]];
}

$stylesheet = preg_replace_callback('/\$GLOBALS\[\'(.*?)\'\]/', 'lookup_global',
    $stylesheet);

$t = new XML_process_stylesheet($stylesheet);
$stylesheet = $t->buf();

if ($debug) {
    print "DOCUMENT:";
    print "<pre>"; print preg_replace("/</", "&lt;", $document); print "</pre>";
    print "<hr>";
    print "STYLESHEET";
    print "<pre>"; print preg_replace("/</", "&lt;", $stylesheet); print "</pre>";
}

//
// ================== Execute the stylesheet
//

$tmpfile = dirname($stylesheetpath) . "/" . getmypid() . ".xsl";

$tf = fopen($tmpfile, "w");
fputs($tf, $stylesheet);
fclose($tf);

$arguments = array(
    '/_xml' => $document,
    '/_xsl' => $stylesheet
    );

$parameters = array(
    'root' => $root,
    'portpath' => $dir,
    'lang' => $lang,
    'tkl_authenticated' => $tkl_authenticated,
    'tkl_bin' => preg_replace("'/[^/]*$'", "", $_SERVER['SCRIPT_NAME']),
    'tkl_cwd' => preg_replace("'/(.*)/.*$'", '\1', $_SERVER['PATH_INFO']),
    'tkl_fn' => preg_replace("'^.*/'", "", $_SERVER['PATH_INFO']),
    'tkl_docroot' => $_SERVER['DOCUMENT_ROOT'],
    );

foreach ($_REQUEST as $key => $value) {
    if (!is_array($value) && !defined($parameters[$key])) {
        if (preg_match('/^utf8_/', $key)) {
            $tname = preg_replace('/^utf8_/', '', $key);
            $parameters[$tname] = $value;
        }
        else {
            $parameters[$key] = utf8_encode($value);
        }
    }
}

function xslt_trap_error($parser, $errorno, $level, $fields) {
    $m = "Error Number $errorno, Level $level, Fields;\n";
    if(is_array($fields)) {
	while(list($key, $value) = each($fields)) {
	    $m .= "<br> $key => $value\n";
	}
    } else {
	$m .= "$fields";
    }
    echo "<p><b>Error</b>: $m";
}

function scheme_tkl_file($scheme, $path) {
    global $debug;
    global $root;
    $opath = $path;

    $path = preg_replace("/^./", "", $path); // Remove leading /
    if (preg_match("/^\//", $path)) {
	$absolute = $_SERVER['DOCUMENT_ROOT'];
	$path = $_SERVER['DOCUMENT_ROOT'] . $root . $path;
    }
    if ($debug)
	print "<hr>Loading file, scheme is $scheme, path is $opath => $path<p>";
    $doc = join('', file($path));
    $t = new XML_process_stylesheet($doc);
    $doc = $t->buf();
    return $doc;
}

function scheme_tkl_search($scheme, $path) {
    if (!preg_match('/\/\/((.*?)(:([0-9]*))?(\/[^\?]*))(\?(.*))?/', $path,
								$match)) {
	print "Malformed path for tkl-search";
	return '';
    }
    $target = $match[1];
    $args = $match[7];
    $att = array('target' => $target, 'wrap' => 1);
    if ($args) {
	foreach (explode('&', $args) as $arg) {
	    preg_match("/^([^=]*)=(.*)$/", $arg, $match);
	    $name = $match[1];
	    $value = $match[2];
	    if (!$name || !$value) {
		print "Bad attribute to search: $arg";
		return '';
	    }
	    $att[$name] = $value;
	}
    }
    return "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n" . function_search($att);
}

// Read an argument string and parse into (hopefully) SOAP-friendly structures.
// Each param is made a member of &array
// aa       => $array[] = aa
// aa=bb    => $array[aa] = bb
// aa.bb    => $array[aa][] = bb
// aa.bb=cc => $array[aa][bb] = cc
// .aa=bb   => $array[top][aa] = bb
// .        => $array[top+1] = array()
// .aa=bb&.cc=dd => array('aa' => 'bb', 'cc' => 'dd')
function grok_arg(&$array, $arg) {
    if (preg_match("/^([^.=]*)\.(.*)$/", $arg, $match)) { // complex
	$head = $match[1];
	$tail = $match[2];
	if ($head == '') {
	    if ($tail == '')
		$head = sizeof($array);
	    else
		$head = max(sizeof($array) - 1, 0);
	}
	if (!is_array($array[$head]))
	    $array[$head] = array();
	if ($tail)
	    grok_arg($array[$head], $tail);
    }
    else { // primitive
	if (!preg_match("/^([^=]*)(=(.*))?$/", $arg, $mmatch))
	    return;
	if ($mmatch[2]) {
	    $name = $mmatch[1];
	    $value = $mmatch[3];
	}
	else { // no index given
	    $name = sizeof($array);
	    $value = $mmatch[1];
	}
	$array[$name] = $value;
    }
}

function scheme_tkl_soap($scheme, $path) {
    global $root;
    $fun = '';

    if (!preg_match("'^([^?]*)(\?(.+))?'", $path, $match)) {
	print "Malformed path for tkl-search";
	return '';
    }
    $wsdl = $match[1];
    $args = $match[3];
    $wsdl = preg_replace("/^./", "", $wsdl); // Remove leading /
    if (preg_match("/^\//", $wsdl)) {
	$wsdl = $_SERVER['DOCUMENT_ROOT'] . $root . $wsdl;
    }
    $att = array('target' => $target, 'wrap' => 1);
    $arg_a = array();
    if ($args) {
	foreach (explode('&', $args) as $arg) {
	    if (preg_match("/^tkl:fun=(.*)$/", $arg, $match))
		$fun = $match[1];
	    else
		grok_arg($arg_a, $arg);
	}
    }

    if (!$fun)
	die("SOAP call must specify tkl:fun argument");

    $soap = new soapclient($wsdl, 'wsdl');
    if (!($res = $soap->call($fun, $arg_a)))
	return '<?xml version="1.0" encoding="UTF-8" ?><error>NOTHING</error>';

   $rtext = $soap->response;   // this is probably not technically part of the nusoap API
   $rtext = preg_replace("/^.*?\s*\n\s*\n/s", "", $rtext);
   return $rtext;
}

function scheme_tkl_args($scheme, $path) {
    $res = "<args>";
    foreach ($_REQUEST as $key => $val) {
	if (is_array($val)) {
	    foreach ($val as $subkey => $subval)
	        $res .= "<$key i=\"$subkey\">$subval</$key>";
	}
	else
	    $res .= "<$key>$val</$key>";
    }
    $res .= "</args>";
    return $res;
}

function scheme_tkl_cmd($scheme, $path) {
    $handlers = array(
        'tkl-path' => 'function_path',
	'tkl-find' => 'function_find',
	'tkl-mail' => 'function_mail',
    );
    $res = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
    $res .= "<$scheme>";
	if (!preg_match('/\/*(.*?)(\?(.*))?/', $path, $match)) {
	    print "Malformed path for $scheme";
	    return '';
	}
	$target = $match[1];
	$args = $match[3];
	$att = array('target' => $target);
	if ($args) {
	    foreach (explode('&', $args) as $arg) {
		list ($name, $value) = explode('=', $arg);
		if (!$name || !$value) {
		    print "Bad attribute to $scheme: $arg";
		    return '';
		}
		$att[$name] = $value;
	    }
	}
	$res .= $handlers[$scheme]($att);
    $res .= "</$scheme>";
    return $res;
}

function scheme_handler($xsl, $scheme, $path) {
    global $debug;
    $handlers = array(
        'tkl-file' => 'scheme_tkl_file',
	'tkl-args' => 'scheme_tkl_args',
	'tkl-time' => 'scheme_tkl_cmd',
	'tkl-find' => 'scheme_tkl_cmd',
	'tkl-path' => 'scheme_tkl_cmd',
	'tkl-mail' => 'scheme_tkl_cmd',
	'tkl-soap' => 'scheme_tkl_soap',
	'tkl-search' => 'scheme_tkl_search',
    );

    if (!$handlers[$scheme]) {
	print "<p><b>Error: Unknown scheme $scheme:</b></p>";
	return '';
    }

    $res = $handlers[$scheme]($scheme, $path);

    if ($debug) {
        print "<hr>";
	print "doc-handler: $scheme:$path ==><br>";
        print "<pre>"; print preg_replace("/</", "&lt;", $res); print "</pre>";
    }

    return $res;
}

$xsl = xslt_create();
xslt_set_base($xsl, "file://$tmpfile");
xslt_set_error_handler($xsl, "xslt_trap_error");
xslt_set_scheme_handlers($xsl, array('get_all' => 'scheme_handler'));
$result = xslt_process($xsl, 'arg:/_xml', 'arg:/_xsl', NULL, $arguments,
    $parameters);
xslt_free($xsl);
unlink($tmpfile);

if (!$result)
    die("error");

if ($debug) {
    print "<hr>RESULT<p><pre>"; print preg_replace("/</", "&lt;", $result); print "</pre>";
} else
    print $result;

// -----------------------------------------------------------------
// Utility stuff -- move everything below this to module files

function function_time($att) {
    if ($att['format']) {
	return strftime($att['format']);
    }
    else {
	return time();
    }
}

function function_path($att) {
    $res = '';
    $dirp = '';

    for ($dirp = './'; ; $dirp .= '../') {
	if (file_exists($dirp . 'index.tkl')) {
	    $res = "<step path='$dirp'>" .
		grok_file(join('', file($dirp . 'index.tkl')), $att) .
	        "</step>\n" .
		$res;
	}
	if (file_exists($dirp . 'tkl.config'))
	    break;
    }

    return $res;
}

function find_files($path, $location, $att, $absolute = '', $level = 0) {
    global $root;

    if (preg_match("/^\//", $path)) {
	$location = $root;
	$absolute = $_SERVER['DOCUMENT_ROOT'];
	$path = preg_replace("/^./", "", $path);
    }
    if (!preg_match("/([^\/]*)(\/(.+))?/", $path, $match))
	return '';
    $local = $match[1];
    $tail = $match[3];
    if (preg_match("/[*?]/", $local)) {
	// Convert glob pattern to regexp
	$local = preg_replace('/\./', '\\.', $local);
	$local = preg_replace('/\?/', '.', $local);
	$local = preg_replace('/\*/', '.*', $local);
	if (!($d = @opendir($absolute . $location)))
	    return '';
	$tmpbuf = '';
	while ($f = readdir($d)) {
	    if (preg_match("/^$local\$/", $f) && !preg_match('/^\./', $f)) {
		$tpath = $f . ($tail ? "/$tail" : '');
		$tmpbuf .= find_files($tpath, $location, $att, $absolute,
							$level);
	    }
	}
	return $tmpbuf;
    }
    if ($tail)
	return find_files($tail, "$location/$local", $att, $absolute, $level);
    if (@is_dir("$absolute$location/$local")) {
	if ($att['level'] && ($att['level'] < $level + 1))
	    return '';
	else
            return "\n<dir path=\"$absolute$location/$local/\" " .
	        "level=\"" . ($level + 1) . "\" att=\"" . $att['level'] . "\">" .
	        find_files("*", "$location/$local", $att, $aboslute,
								$level + 1) . 
	        "</dir>";
    }
    if (@file_exists("$absolute$location/$local")) {
	if ($att['mask'] && $att['mask'] != $local)
	    return '';
	$fl = join('', file("$absolute$location/$local"));

	// Hidden attribute is set
	if (preg_match("/<[^?!][^>]*hidden=.1./", $fl)) {
	    return '';
	}

	if ($att['type']) {
	    // Divine document element
	    preg_match('/<([^!?][^\s>]*)/', $fl, $match);
	    if ($att['type'] != $match[1])
	       return '';
	}

	if ($local == 'index.tkl') {
	    $local = '';
	}
	return "\n<file path=\"$location/$local\">" .
	    grok_file($fl, $att) .
	    "</file>";
    }
}

function xml_getelement($xml, $element) {
    preg_match_all("/(<$element\s*.*?>\s*.*?\s*<\/$element>)/s", $xml, $res,
   	 PREG_PATTERN_ORDER);
    return $res[1];
}

function xml_getelement_content($xml, $element) {
    preg_match_all("/<$element\s*.*?>(.*?)<\/$element>/s", $xml, $res,
   	 PREG_PATTERN_ORDER);
    return $res[1];
}



// Extract elements from file if specified by select-attribute
function grok_file($fl, $att) {
    $res = '';
    global $lang;
    $alpha_prefix = 'den|det|et|en|der|die|das|the|a|an';

    if ($att['select']) {
	foreach (explode('|', $att['select']) as $field) {
	    $alphasort = 0;
	    if (preg_match("'.*/@alphasort'", $field)) {
		$alphasort = 1;
		$field = preg_replace("'/.*$'", "", $field);
	    }
	    foreach (xml_getelement($fl, $field) as $elem) {
		if (!preg_match('/<[^>]*xml:lang\s*=\s*["\'](.*?)["\']/',
		        $elem, $match) || $match[1] == $lang) {
		    if ($alphasort) {
			$lower = strtolower($elem);
			preg_match("/<.*?>(($alpha_prefix)\s+)?(.*?)<\//", $lower, $match);
			$tmp = preg_replace("/aa/", "}", $match[3]);
			$tmp = preg_replace("/[]/", "{", $tmp);
			$tmp = preg_replace("/[]/", "|", $tmp);
			$tmp = preg_replace("/[]/", "}", $tmp);
			$elem = preg_replace("/([\s>])/", " alphasort=\"$tmp\"\\1", $elem, 1);
		    }
		    $res .= $elem;
		}
	    }
	}
    }
    return $res;
}

function function_find($att) {
    if (!($path = $att['path'])) 
	return "<error>must specify path attr</error>";
    return find_files($path, ".", $att);
}

function alphasort_callback($match) {
    
    $alpha_prefix = 'den|det|et|en|der|die|das|the|a|an';
    $lower = strtolower($match[3]);
    $tmp = preg_replace("/^($alpha_prefix)\s+/", "", $lower);
    $tmp = preg_replace("/aa/", "}", $tmp);
    $tmp = preg_replace("/[]/", "{", $tmp);
    $tmp = preg_replace("/[]/", "|", $tmp);
    $tmp = preg_replace("/[]/", "}", $tmp);
    return "<$match[1] alphasort=\"$tmp\">$match[3]</title>";
}

function function_search($att) {
    global $document;
    global $debug;

    if (!$att['target'])
	return '<error>No target specified</error>';
    $number = $att['number'] OR $number = 0;
    $start = $att['start'] OR $start = 1;
    $syntax = $att['syntax'] OR $syntax = 'xml';
    $wrap = $att['wrap'];
    $res = '';

    //return '<server host="bagel/gils" status="1"><hits>0</hits></server>';

    $y = yaz_connect($att['target'], array('persistent' => FALSE));
    yaz_range($y, $start, $number);
    yaz_syntax($y, $syntax);

    if ($att['cclquery']) {  // Find CCL fields and parse query
	$cclfields = array();

	if ($arr = xml_getelement($document, 'cclfield')) {
	    foreach ($arr as $elem) {
		if (!preg_match("'<cclfield\s.*fieldname=\"(.*?)\".*>(.*?)<\/cclfield>'", $elem, $match))
		    continue;
		$fieldn = $match[1];
		$val = $match[2];
		$cclfields[$fieldn] = $val;
	    }
	}
	if ($cclfields) {
	    yaz_ccl_conf($y, $cclfields);
	    if (!yaz_ccl_parse($y, $att['cclquery'], $out))
		return '<bad_query/>';
	    $query = $out['rpn'];
	}
	list($pqfprefix) = xml_getelement_content($document, 'pqfprefix');
	list($pqfsuffix) = xml_getelement_content($document, 'pqfsuffix');
	if ($pqfprefix) $query = "$pqfprefix $query";
	if ($pqfsuffix) $query = "$query $pqfsuffix";

	if ($debug)
	    print "<p/>CCL-Query='" . $att['cclquery'] . "', RPN=$query<br/>";
    }
    else
	$query = $att['query'];

    if (!$query)
	return '<no_query/>';

    yaz_search($y, 'rpn', utf8_decode($query));
    yaz_wait();
    if (yaz_errno($y))
	return "<error code=\"" . yaz_errno($y) . "\">" . yaz_error($y) .
   	    "</error>";

    $hits = yaz_hits($y);

    if ($wrap)
	$res .= "<search>";
    $res .= "<start>$start</start><number>$number</number>";

    $res .= "<server url=\"". $att['target'] . "\" status=\"1\">";
    $res .= "<hits>$hits</hits>";

    $end = $start + $number - 1;
    if ($end > $hits)
	$end = $hits;
    $res .= "<end>$end</end>";

    for ($recno = $start, $count = 1;
                $recno <= $hits && $count <= $number; $recno++, $count++) {
	$res .= "<record offset=\"$recno\"";
	$rec = preg_replace("/<\?.*?\?>/", "", yaz_record($y, $recno, 'string'));
	$rec = preg_replace("/&/", "&amp;", $rec);
	if (!$rec)
	    $res .= " error=\"No record at that position\">";
	else {
	    if ($att['alphasort']) {
		$fld = $att['alphasort'];
		 
		$rec = preg_replace_callback("/<($fld(\s[^>]*))?>(.*?)<\/.*?>/", 'alphasort_callback', $rec);
	    }
	    $res .= ">$rec";
	}
	$res .= "</record>";

    }

    $res .= '</server>';
    if ($wrap)
	$res .= "</search>";

    $t = new XML_process_stylesheet($res);
    $res = $t->buf();
    $res = preg_replace("/&/", "&amp;", $res);
    return $res;
}

function function_mail($att) {
    if (!$att['to'])
	die("No recipient address for mail");
    $result = @mail($att['to'], utf8_decode($att['subject']), utf8_decode($att['message']));
    if ($result)
	return "<status>OK</status>";
    else
	return "<error>Failed to send mail</error>";
}

function function_session_var($att) {
    if (!$att['name'])
	die("No 'name' given for session-var");
    $name = $att['name'];

    if ($_SESSION[$name] && !$_REQUEST[$name])
	$_REQUEST[$name] = $_SESSION[$name];
    elseif (!$_REQUEST[$name] && $att['default'])
	$_REQUEST[$name] = $att['default'];
    $_SESSION[$name] = $_REQUEST[$name];
}

// This scans any XML document for commands (elements at the top level in the
// http://www.indexdata.dk/TKL namsepace), and calls the associated functions.
// The results are placed in the elements themselves.
class XML_process_stylesheet {

    var $outbuf;      // buffer for XML output
    var $level;       // level in the hierarchy
    var $command_ns;  // prefix for commands
    var $current_tag;
    var $stack;
    var $stackp;
    var $skip;

    var $fun = array (
        'search' => 'function_search',
	'time' => 'function_time',
	'find' => 'function_find',
	'path' => 'function_path',
	'session-var' => 'function_session_var',
	);

    function XML_process_stylesheet($xml) {
	$this->outbuf = '';
	$this->level = -1;
	$this->comand_ns = undef;
	$this->stack = array();
	$this->stackp = -1;
	$this->skip = 0;
	$xp = xml_parser_create();
	xml_parser_set_option($xp, XML_OPTION_CASE_FOLDING, 0);
	xml_set_object($xp, &$this);
	xml_set_element_handler($xp, "elem_begin", "elem_end");
	xml_set_default_handler($xp, "default_handler");
	xml_set_character_data_handler($xp, "character_data_handler");
	if (!xml_parse($xp, $xml)) {
	    print "Error parsing stylesheet (line #" . xml_get_current_line_number($xp) . ": " . xml_error_string(xml_get_error_code($xp)) . "; Current element &lt;" . $this->current_tag . "&gt;";
	    exit();
        }
	xml_parser_free($xp);
    }

    function default_handler($xp, $data) {
	if ($this->skip)
	    return;
	$this->outbuf .= "$data";
    }

    function character_data_handler($xp, $data) {
	if ($this->skip)
	    return;
        $tmp = preg_replace("/&/", '&amp;', $data);
        $tmp = preg_replace("/</", '&lt;', $tmp);
        $tmp = preg_replace("/>/", '&gt;', $tmp);
	$this->outbuf .= "$tmp";
    }

    function elem_end($xp, $elem) {
	$this->level--;
	$this->stackp--;
	if (!$this->skip)
	    $this->outbuf .= "</$elem>";
	if ($this->stackp >= 0)
	    $this->skip = $this->stack[$this->stackp]['skip'];
    }

    // The meat. Dump elements. Make note of namespace prefixes for
    // the 'commands'. Execute commands when found.
    function elem_begin($xp, $elem, $att) {
	global $lang;

	$skip = 0;
	$command_ns = $this->command_ns;
	$this->level++;
	$this->current_tag = $elem;
	if ($this->stackp >= 0 && $this->stack[$this->stackp]['skip'])
	    $skip = 1;
	elseif ($att['xml:lang'] && $att['xml:lang'] != $lang)
	    $skip = 1;
	$this->skip = $skip;
	$this->stackp++;
	$this->stack[$this->stackp] = array('skip' => $skip, 'tag' => $elem);
	if ($skip)
	    return;
	$this->outbuf .= "<$elem";
	if ($att) {
	    foreach ($att as $k => $v) {
		$tmp = preg_replace("/&/", '&amp;', $v);
		$tmp = preg_replace("/</", '&lt;', $tmp);
		$tmp = preg_replace("/>/", '&gt;', $tmp);
	        $this->outbuf .= " $k=\"$tmp\"";

		if ($this->level == 0 || $this->level == 1) {
		    if (preg_match("/^xmlns(:(.+))?/", $k, $match) &&
			               $v == "http://www.indexdata.dk/TKL") {
			$command_ns = $match[2];
			if ($this->level == 0) // global namespace
			    $this->command_ns = $command_ns;
		    }
		}
	    }
	}
	$this->outbuf .= ">";

	if ($this->level == 1 && $command_ns && preg_match(
	       "/^$command_ns:(.*)/", $elem, $res)) {
	    $fun_name = $res[1];
	    if ($this->fun[$fun_name]) {
		$this->outbuf .= $this->fun[$fun_name]($att);
	    } else {
		$this->outbuf .= "<error>No such function: '$fun_name'</error>";
	    }
	}
    }

    function buf() {
	return $this->outbuf;
    }
}

?>
